/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.task;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.ProgressBar;

import com.thingspace.files.R;

import java.io.IOException;
import java.lang.ref.WeakReference;
import java.net.URL;
import java.net.URLConnection;

import javax.net.ssl.HttpsURLConnection;

/**
 * Get full file
 */
public class GetFileTask extends AsyncTask<String, Void, Bitmap> {

    private final WeakReference<ImageView> imageViewWeakReference;
    private WeakReference<Context> contextWeakReference;
    private String accessToken, path;
    private ProgressBar progressBar;

    /**
     * Get full file from the cloud.
     *
     * @param path        file path returned with fullview response
     * @param accessToken access token to connect to Thingspace api gateway
     * @param imageView   image view to bind the bitmap with
     * @param context     Context to read server url from the resource file
     * @param progressBar ProgressBar object to display/hide the progress
     */
    public GetFileTask(String path, String accessToken, ImageView imageView, Context context, ProgressBar progressBar) {
        this.path = path;
        this.accessToken = accessToken;
        imageViewWeakReference = new WeakReference<>(imageView);
        contextWeakReference = new WeakReference<>(context);
        this.progressBar = progressBar;
    }

    @Override
    protected Bitmap doInBackground(String... params) {
        return getFile(contextWeakReference.get().getString(R.string.THINGSPACE_SERVER_URL) + "/cloud/1/files" + path.replace(" ", "%20"));
    }

    private Bitmap getFile(String url) {
        try {

            if (isCancelled()) {
                return null;
            }

            URLConnection urlConnection = new URL(url).openConnection();
            HttpsURLConnection httpURLConnection = (HttpsURLConnection) urlConnection;
            httpURLConnection.addRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            httpURLConnection.addRequestProperty("Authorization", "Bearer " + accessToken);
            httpURLConnection.addRequestProperty("Content-Language", "en-US");
            httpURLConnection.addRequestProperty("Accept", "application/json");
            httpURLConnection.setRequestMethod("GET");

            Log.e(getClass().getSimpleName(), "response: " + httpURLConnection.getResponseCode());

            if (httpURLConnection.getResponseCode() == HttpsURLConnection.HTTP_OK) {
                try {
                    BitmapFactory.Options options = new BitmapFactory.Options();
                    options.inSampleSize = 2;
                    if (isCancelled()) {
                        return null;
                    }

                    return BitmapFactory.decodeStream(httpURLConnection.getInputStream(), null, options);
                } catch (IOException e) {
                    e.printStackTrace();
                    return null;
                }
            } else if (httpURLConnection.getResponseCode() == HttpsURLConnection.HTTP_MOVED_PERM ||
                    httpURLConnection.getResponseCode() == HttpsURLConnection.HTTP_MOVED_TEMP) {
                Log.e(getClass().getSimpleName(), "302 - Redirecting");
                return getFile(httpURLConnection.getHeaderField("Location"));
            }
        } catch (Exception e) {
            Log.e(getClass().getSimpleName(), "Exception retrieving full photo: " + e);
        }
        return null;
    }

    @Override
    protected void onPostExecute(Bitmap bitmap) {
        super.onPostExecute(bitmap);
        progressBar.setVisibility(View.GONE);
        if (imageViewWeakReference != null || bitmap != null) {
            ImageView imageView = imageViewWeakReference.get();
            if (imageView != null && bitmap != null) {
                imageView.setImageBitmap(bitmap);
            }
        } else {
            Log.e(getClass().getSimpleName(), "imageview weak reference null");
        }
    }
}
